"use strict";
/**
 * Copyright 2020 Valentin Draganescu
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var metadata_storage_1 = require("../metadata/metadata-storage");
var logger_1 = require("../logger");
var validateIsRequired = function (propMeta, propertyValue, Model, modelKey) {
    if (propMeta.isRequired && !propertyValue) {
        return Model.name + "." + modelKey + " is required";
    }
    return null;
};
var validateIsCorrectType = function (propType, propMeta, Model, modelKey) {
    if (propType !== propMeta.type) {
        return Model.name + "." + modelKey + " should be of type " + propMeta.type;
    }
    return null;
};
var validateMinSize = function (propMeta, propType, property, Model, modelKey) {
    if (propMeta.minSize) {
        if (propType === "string" || (propType === "object" && Array.isArray(property))) {
            if (property.length < propMeta.minSize) {
                return Model.name + "." + modelKey + " should have a minimum size of " + propMeta.minSize;
            }
        }
        if (propType === "number" || propType === "bigint") {
            if (property < propMeta.minSize) {
                return Model.name + "." + modelKey + " should be at least " + propMeta.minSize;
            }
        }
    }
    return null;
};
var validateFormat = function (propMeta, property, Model, modelKey) {
    if (propMeta.format) {
        var regex = new RegExp(propMeta.format);
        var isCorrectFormat = regex.test(property);
        if (!isCorrectFormat) {
            return Model.name + "." + modelKey + " should have the format " + propMeta.format;
        }
    }
    return null;
};
var validateMaxSize = function (propMeta, propType, property, Model, modelKey) {
    if (propMeta.maxSize) {
        if (propType === "string" || (propType === "object" && Array.isArray(property))) {
            if (property.length > propMeta.maxSize) {
                return Model.name + "." + modelKey + " should have a maximum size of " + propMeta.minSize;
            }
        }
        if (propType === "number" || propType === "bigint") {
            if (property > propMeta.maxSize) {
                return Model.name + "." + modelKey + " should be at most " + propMeta.minSize;
            }
        }
    }
    return null;
};
var validateObject = function (propMeta, propertyVallue) {
    var errors = [];
    if (propMeta.objectType) {
        errors.push.apply(errors, Validator.validate(propertyVallue, propMeta.objectType));
    }
    else {
        var message = "No entity defined for object " + JSON.stringify(propertyVallue);
        console.error(message);
        errors.push({ message: message });
    }
    return errors;
};
var validateRequiredProperties = function (property, propMeta, Model, modelKey) {
    var errors = [];
    var error;
    var propType = typeof property;
    var typeError = validateIsCorrectType(propType, propMeta, Model, modelKey);
    if (typeError) {
        errors.push({ message: typeError });
    }
    if (!typeError) {
        error = validateMinSize(propMeta, propType, property, Model, modelKey);
        if (error) {
            errors.push({ message: error });
        }
        error = validateMaxSize(propMeta, propType, property, Model, modelKey);
        if (error) {
            errors.push({ message: error });
        }
        error = validateFormat(propMeta, property, Model, modelKey);
        if (error) {
            errors.push({ message: error });
        }
    }
    return errors;
};
var Validator = /** @class */ (function () {
    function Validator() {
    }
    Validator.validate = function (body, Model) {
        logger_1.Logger.log("Validating", Model.name, JSON.stringify(body));
        var errors = [];
        if (!body) {
            errors.push({
                message: Model.name + " is required"
            });
            return errors;
        }
        var metadata = metadata_storage_1.getMetadataStorage();
        var entityMeta = metadata.entities[Model.name];
        if (!entityMeta) {
            var message = "Entity " + Model.name + " is not registered with the router, did you forget to decorate " + Model.name + " with @DocModel()?";
            console.error(message);
            errors.push({ message: message });
        }
        var modelKeys = Object.keys(entityMeta);
        for (var _i = 0, modelKeys_1 = modelKeys; _i < modelKeys_1.length; _i++) {
            var modelKey = modelKeys_1[_i];
            var propMeta = entityMeta[modelKey];
            var propertyValue = body[modelKey];
            logger_1.Logger.log("Validating " + Model.name + "." + modelKey);
            if (typeof propertyValue === "object") {
                logger_1.Logger.log("Property is an object");
                errors.push.apply(errors, validateObject(propMeta, propertyValue));
            }
            else {
                logger_1.Logger.log("Property is a primitive with value", propertyValue);
                if (propMeta) {
                    var error = validateIsRequired(propMeta, propertyValue, Model, modelKey);
                    if (error) {
                        errors.push({ message: error });
                    }
                    if (propertyValue) {
                        errors.push.apply(errors, validateRequiredProperties(propertyValue, propMeta, Model, modelKey));
                    }
                }
            }
        }
        return errors;
    };
    return Validator;
}());
exports.Validator = Validator;
//# sourceMappingURL=validator.js.map